'use strict';

var qs = require('qs');

var _require = require('../../helpers'),
    isObject = _require.isObject;

module.exports = function modifyLoaders(configuration, pluginOptions) {
    var aliases = getBabelLoaderAliases(pluginOptions);
    var loadersParams = getBabelLoaders(configuration, aliases);
    loadersParams.forEach(function (loaderParams) {
        return addQueryParams(loaderParams, aliases);
    });

    return loadersParams.length > 0;
};

function getBabelLoaderAliases(pluginOptions) {
    return [/(?:^|!)babel(?:(?:-\w+)?-loader)?$/i].concat(pluginOptions.get('aliases'));
}

function getBabelLoaders(configuration, aliases) {
    return getLoaders(configuration).filter(function (loaderParams) {
        return matchLoaders(loaderParams, aliases);
    });
}

function getLoaders(configuration) {
    var loaders = isObject(configuration.module) && configuration.module.loaders;

    return Array.isArray(loaders) ? loaders : [];
}

function matchLoaders(loaderParams, aliases) {
    var loaders = [];
    if (typeof loaderParams.loader === 'string') {
        loaders = loaderParams.loader.split('!');
    } else if (Array.isArray(loaderParams.loaders)) {
        loaders = loaderParams.loaders;
    }

    return extractLoaderNames(loaders).some(function (loaderName) {
        return matchLoaderNameByAliases(loaderName, aliases);
    });
}

function extractLoaderNames(loaderStrings) {
    return loaderStrings.reduce(function (result, loaderString) {
        var loaderName = extractLoaderName(loaderString);
        if (loaderName) {
            result.push(loaderName);
        }

        return result;
    }, []);
}

function extractLoaderName() {
    var loaderString = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

    return loaderString.split('?')[0];
}

function extractLoaderQuery() {
    var loaderString = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

    var querySignIndex = loaderString.indexOf('?');
    var result = '';
    if (querySignIndex > -1) {
        result = loaderString.substring(querySignIndex + 1);
    }

    return result;
}

function matchLoaderNameByAliases(loaderName, aliases) {
    loaderName = loaderName.toLowerCase();

    return aliases.some(function (alias) {
        if (alias instanceof RegExp) {
            return alias.test(loaderName);
        }

        return alias.toLowerCase() === loaderName;
    });
}

function addQueryParams(loaderParams, aliases) {
    if (loaderParams.query) {
        loaderParams.query = injectQueryParamsToLoaderQuery(loaderParams.query);

        return;
    }

    var loaderString = typeof loaderParams.loader === 'string' ? loaderParams.loader : null;
    if (loaderString) {
        loaderParams.loader = injectQueryParamsToLoaderString(loaderString, aliases);
    } else if (Array.isArray(loaderParams.loaders)) {
        loaderParams.loaders = injectQueryParamsToLoadersArray(loaderParams.loaders, aliases);
    }
}

function injectQueryParamsToLoaderQuery(query) {
    var queryObject = getQueryObject(query);
    injectPluginsParams(queryObject);

    return qs.stringify(queryObject, { arrayFormat: 'brackets', encode: false });
}

function getQueryObject(query) {
    if (typeof query === 'string') {
        query = parseQueryString(query);
    }

    return query;
}

function parseQueryString(queryString) {
    if (queryString.charAt(0) === '{' && queryString.charAt(queryString.length - 1) === '}') {
        return JSON.parse(queryString);
    }

    return qs.parse(queryString, { allowDots: true });
}

function injectPluginsParams(query) {
    var helpersPluginRegexp = /(?:^|babel-plugin-)external-helpers$/i;
    var plugins = Array.isArray(query.plugins) ? query.plugins : [];
    if (!plugins.some(function (pluginName) {
        return helpersPluginRegexp.test(pluginName);
    })) {
        plugins.push('external-helpers');
    }
    query.plugins = plugins.filter(Boolean);
}

function injectQueryParamsToLoaderString(loaders, aliases) {
    return injectQueryParamsToLoadersArray(loaders.split('!'), aliases).join('!');
}

function injectQueryParamsToLoadersArray(loaderStrings, aliases) {
    return loaderStrings.map(function (loaderString) {
        var loaderName = extractLoaderName(loaderString);
        if (!matchLoaderNameByAliases(loaderName, aliases)) {
            return loaderString;
        }

        var queryString = injectQueryParamsToLoaderQuery(extractLoaderQuery(loaderString));

        return loaderName + '?' + queryString;
    });
}